import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Icon, Modal } from 'antd'

import Utils from '@/utils/utils.js'
import zhCN from '@/locales/zh-CN/model.js'
import enUS from '@/locales/en-US/model.js'
import { getChartViewForm } from '@/templates/zshare/formconfig'

import ChartForm from './chartform'
import DragChartView from './dragchartview'
import './index.scss'

const { confirm } = Modal

// **悲观者往往正确，乐观者往往成功
class ChartGroupComponent extends Component {
  static propTpyes = {
    config: PropTypes.object,         // 菜单配置信息
    sysRoles: PropTypes.array,        // 角色列表，黑名单使用
    updatechartgroup: PropTypes.func  // 图表更新
  }

  state = {
    dict: (!localStorage.getItem('lang') || localStorage.getItem('lang') === 'zh-CN') ? zhCN : enUS,
    chartlist: null,      // 图表集
    card: null,           // 编辑中元素
    formlist: null,       // 表单信息
    modaltype: '',        // 模态框控制
    chartview: null       // 活动图表
  }

  /**
   * @description 图表初始化
   */
  UNSAFE_componentWillMount () {
    this.setState({
      chartlist: fromJS(this.props.config.charts).toJS(),
      chartview: this.props.config.charts[0].uuid
    })
  }

  /**
   * @description 图表信息修改时更新 chartlist
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { chartlist } = this.state

    if (nextProps.config && nextProps.config.charts && !is(fromJS(nextProps.config.charts), fromJS(chartlist))) {
      this.setState({
        chartlist: fromJS(nextProps.config.charts).toJS()
      })
    }
  }

  /**
   * @description 图表顺序调整，或拖拽添加
   */
  handleList = (list) => {
    const { config } = this.props

    this.setState({chartlist: list})
    this.props.updatechartgroup({...config, charts: list}, this.state.chartview)
  }

  /**
   * @description 取消保存，关闭模态框
   */
  editModalCancel = () => {
    this.setState({
      card: null,
      modaltype: ''
    })
  }

  /**
   * @description 添加或修改图表
   */
  handleChart = (item) => {
    let _type = 'editChart'
    if (!item) {
      _type = 'addChart'
      item = {
        uuid: Utils.getuuid(),
        label: '',
        title: '',
        chartType: 'line',
        icon: 'line-chart',
        Hide: 'false',
        blacklist: []
      }
    }

    this.setState({
      card: item,
      modaltype: _type,
      formlist: getChartViewForm(item, this.props.sysRoles)
    })
  }

  /**
   * @description 图表信息修改后，提交保存
   */
  submitChart = () => {
    const { config } = this.props
    const { modaltype } = this.state
    let _chartlist = fromJS(this.state.chartlist).toJS()
    let _chartview = this.state.chartview

    this.chartFormRef.handleConfirm().then(res => {
      if (modaltype === 'addChart') {
        _chartlist.push(res)
        _chartview = res.uuid
      } else {
        _chartlist = _chartlist.map(item => {
          if (item.uuid === res.uuid) {
            if (!is(fromJS(item), fromJS(res))) {
              let _element = document.getElementById(res.uuid)
              if (_element) {
                _element.innerHTML = ''
              }
            }
            return res
          }
          return item
        })
      }

      this.setState({
        card: null,
        chartview: _chartview,
        modaltype: '',
        chartlist: _chartlist
      })

      this.props.updatechartgroup({...config, charts: _chartlist}, _chartview)
    })
  }

  /**
   * @description 图表展开收起切换
   */
  onChartChange = (e) => {
    const { config } = this.props
    e.stopPropagation()

    this.props.updatechartgroup({...config, expand: !config.expand}, this.state.chartview)
  }

  /**
   * @description 图表删除
   */
  deletechart = (plot) => {
    const { config } = this.props
    const { dict } = this.state
    let _this = this

    confirm({
      content: dict['model.confirm'] + dict['model.delete'] + ` ${plot.title} ？`,
      okText: dict['model.confirm'],
      cancelText: dict['header.cancel'],
      onOk() {
        let _chartlist = fromJS(_this.state.chartlist).toJS()
        let _chartview = this.state.chartview

        _chartlist = _chartlist.filter(item => item.uuid !== plot.uuid)

        if (_chartview === plot.uuid) {
          _chartview = _chartlist[0].uuid
        }

        _this.setState({
          chartlist: _chartlist,
          chartview: _chartview
        })
        _this.props.updatechartgroup({...config, charts: _chartlist}, _chartview)
      },
      onCancel() {}
    })
  }

  changetabview = (id) => {
    const { config } = this.props

    this.setState({
      chartview: id
    })

    this.props.updatechartgroup(config, id)
  }


  render() {
    const { config } = this.props
    const { dict, chartlist, modaltype, card, chartview } = this.state

    return (
      <div className="model-table-chartview-list">
        <Icon type="plus" onClick={() => this.handleChart()} />
        {chartlist.length > 1 ? <Icon type={config.expand ? 'up' : 'down'} onClick={this.onChartChange} /> : null}
        {chartlist.length > 1 ? <DragChartView
          activeKey={chartview}
          list={chartlist}
          expand={config.expand}
          handleList={this.handleList}
          changetabview={this.changetabview}
          handleMenu={this.handleChart}
          deleteMenu={this.deletechart}
        /> : null}
        {/* 合并列编辑 */}
        <Modal
          title={modaltype === 'addChart' ? '图表-添加' : '图标-编辑'}
          visible={modaltype === 'addChart' || modaltype === 'editChart'}
          width={750}
          maskClosable={false}
          onOk={this.submitChart}
          onCancel={this.editModalCancel}
          destroyOnClose
        >
          <ChartForm
            dict={dict}
            card={card}
            formlist={this.state.formlist}
            inputSubmit={this.submitChart}
            wrappedComponentRef={(inst) => this.chartFormRef = inst}
          />
        </Modal>
      </div>
    )
  }
}

export default ChartGroupComponent